@@ -293,10 +293,9 @@ class BaseSim(ParsObj):
             # Handle other special parameters
             if pars.get('network'):
                 hppar.reset_layer_pars(pars, force=False)
-            location = 'nigeria'
             if pars.get('location'):
                 location = pars['location']
-            pars['birth_rates'], pars['death_rates'] = hppar.get_births_deaths(location=location) # Set birth and death rates
+                pars['birth_rates'], pars['death_rates'] = hppar.get_births_deaths(location=location) # Set birth and death rates
 
             # Call update_pars() for ParsObj
             super().update_pars(pars=pars, create=create)
@@ -1381,47 +1380,38 @@ class BasePeople(FlexPretty):
     @property
     def precin(self):
         '''
-        Boolean array of everyone with whose disease severity level does not meet the threshold for detectable cell changes
+        Boolean array of females with HPV whose disease severity level does not meet the threshold for detectable cell changes
         '''
-        return (self.sev < self.pars['clinical_cutoffs']['precin']).astype(bool)
+        return ((self.sex == 0) & self.infectious & (np.isnan(self.sev) | (self.sev==0))).astype(bool)
 
     @property
     def cin1(self):
         '''
-        Boolean array of everyone with whose disease severity level lies within the thresholds for CIN1-level cell changes
+        Boolean array of females with HPV whose disease severity level lies within the thresholds for CIN1-level cell changes
         '''
-        return ((self.sev >= self.pars['clinical_cutoffs']['precin']) * (
-                 self.sev < self.pars['clinical_cutoffs']['cin1'])).astype(bool)
+        return ((self.sex == 0) * (self.sev > 0) * (self.sev < self.pars['clinical_cutoffs']['cin1'])).astype(bool)
 
     @property
     def cin2(self):
         '''
-        Boolean array of everyone with whose disease severity level lies within the thresholds for CIN2-level cell changes
+        Boolean array of females with HPV whose disease severity level lies within the thresholds for CIN2-level cell changes
         '''
-        return ((self.sev >= self.pars['clinical_cutoffs']['cin1']) * (
+        return ((self.sex == 0) * (self.sev >= self.pars['clinical_cutoffs']['cin1']) * (
                  self.sev < self.pars['clinical_cutoffs']['cin2'])).astype(bool)
 
     @property
     def cin3(self):
         '''
-        Boolean array of everyone with whose disease severity level lies within the thresholds for CIN3-level cell changes
+        Boolean array of females with HPV whose disease severity level lies within the thresholds for CIN3-level cell changes
         '''
-        return ((self.sev >= self.pars['clinical_cutoffs']['cin2']) * (
-                 self.sev < self.pars['clinical_cutoffs']['cin3'])).astype(bool)
-
-    @property
-    def carcinoma(self):
-        '''
-        Boolean array of everyone with whose disease severity level lies within the thresholds for carcinoma in situ
-        '''
-        return (self.sev >= self.pars['clinical_cutoffs']['cin3']).astype(bool)
+        return ((self.sex == 0) * (self.sev >= self.pars['clinical_cutoffs']['cin2'])).astype(bool)
 
     @property
     def cin(self):
         '''
-        Boolean array of everyone with whose disease severity level meets the threshold for detectable cell changes
+        Boolean array of females with HPV whose disease severity level meets the threshold for detectable cell changes
         '''
-        return (self.sev >= self.pars['clinical_cutoffs']['precin']).astype(bool)
+        return (self.sev>0).astype(bool)
 
     def true(self, key):
         ''' Return indices matching the condition '''
