@@ -75,7 +75,7 @@ class Calibration(sc.prettyobj):
 
     '''
 
-    def __init__(self, sim, datafiles, calib_pars=None, genotype_pars=None, hiv_pars=None, fit_args=None, extra_sim_results=None,
+    def __init__(self, sim, datafiles, calib_pars=None, genotype_pars=None, hiv_pars=None, fit_args=None, extra_sim_result_keys=None,
                  par_samplers=None, n_trials=None, n_workers=None, total_trials=None, name=None, db_name=None,
                  keep_db=None, storage=None, rand_seed=None, label=None, die=False, verbose=True):
 
@@ -97,7 +97,7 @@ class Calibration(sc.prettyobj):
         self.calib_pars     = calib_pars
         self.genotype_pars  = genotype_pars
         self.hiv_pars       = hiv_pars
-        self.extra_sim_results = extra_sim_results
+        self.extra_sim_result_keys = extra_sim_result_keys
         self.fit_args       = sc.mergedicts(fit_args)
         self.par_samplers   = sc.mergedicts(par_samplers)
         self.die            = die
@@ -110,16 +110,7 @@ class Calibration(sc.prettyobj):
             self.target_data.append(hpm.load_data(datafile))
 
         sim_results = sc.objdict()
-
         age_result_args = sc.objdict()
-        extra_sim_results = sc.objdict()
-
-        if self.extra_sim_results:
-            for extra_result in self.extra_sim_results:
-                extra_sim_results[extra_result] = sc.objdict()
-            self.extra_sim_results_keys = extra_sim_results.keys()
-        else:
-            self.extra_sim_results_keys = None
 
         # Go through each of the target keys and determine how we are going to get the results from sim
         for targ in self.target_data:
@@ -160,8 +151,8 @@ class Calibration(sc.prettyobj):
                 self.result_args[rkey].name = self.sim.results[rkey].name
                 self.result_args[rkey].color = self.sim.results[rkey].color
 
-        if self.extra_sim_results:
-            for rkey in self.extra_sim_results_keys:
+        if self.extra_sim_result_keys:
+            for rkey in self.extra_sim_result_keys:
                 self.result_args[rkey] = sc.objdict()
                 self.result_args[rkey].name = self.sim.results[rkey].name
                 self.result_args[rkey].color = self.sim.results[rkey].color
@@ -173,11 +164,12 @@ class Calibration(sc.prettyobj):
 
     def run_sim(self, calib_pars=None, genotype_pars=None, hiv_pars=None, label=None, return_sim=False):
         ''' Create and run a simulation '''
-        sim = self.sim.copy()
+        sim = sc.dcp(self.sim)
         if label: sim.label = label
 
         new_pars = self.get_full_pars(sim=sim, calib_pars=calib_pars, genotype_pars=genotype_pars, hiv_pars=hiv_pars)
         sim.update_pars(new_pars)
+        sim.initialize(reset=True, init_analyzers=False) # Necessary to reinitialize the sim here so that the initial infections get the right parameters
 
         # Run the sim
         try:
@@ -394,6 +386,10 @@ class Calibration(sc.prettyobj):
         for key, val in pardict.items():
             if isinstance(val, list):
                 low, high = val[1], val[2]
+                if (len(val)>3):
+                    step = val[3]
+                else:
+                    step=None
                 if key in self.par_samplers:  # If a custom sampler is used, get it now
                     try:
                         sampler_fn = getattr(trial, self.par_samplers[key])
@@ -406,7 +402,7 @@ class Calibration(sc.prettyobj):
                     sampler_key = gname + '_' + key
                 else:
                     sampler_key = key
-                pars[key] = sampler_fn(sampler_key, low, high)  # Sample from values within this range
+                pars[key] = sampler_fn(sampler_key, low, high, step=step)  # Sample from values within this range
 
             elif isinstance(val, dict):
                 sampler_fn = trial.suggest_float
@@ -418,7 +414,17 @@ class Calibration(sc.prettyobj):
                         sampler_key = key + '_' + parkey
                     if isinstance(par_highlowlist, dict):
                         par_highlowlist = par_highlowlist['value']
-                    pars[key][parkey] = sampler_fn(sampler_key, par_highlowlist[1], par_highlowlist[2])
+                        low, high = par_highlowlist[1], par_highlowlist[2]
+                        if (len(par_highlowlist) > 3):
+                            step = par_highlowlist[3]
+                        else: step = None
+                    elif isinstance(par_highlowlist, list):
+                        low, high = par_highlowlist[1], par_highlowlist[2]
+                        if (len(par_highlowlist) > 3):
+                            step = par_highlowlist[3]
+                        else:
+                            step = None
+                    pars[key][parkey] = sampler_fn(sampler_key, low, high, step=step)
 
         return pars
 
@@ -443,10 +449,13 @@ class Calibration(sc.prettyobj):
 
         sim = self.run_sim(calib_pars, genotype_pars, hiv_pars, return_sim=True)
 
-        # Compute fit for sim results and save sim results (TODO: THIS IS BY GENOTYPE FOR A SINGLE TIMEPOINT. GENERALIZE THIS)
+        # Compute fit for sim results and save sim results (TODO: THIS IS FOR A SINGLE TIMEPOINT. GENERALIZE THIS)
         sim_results = sc.objdict()
         for rkey in self.sim_results:
-            model_output = sim.results[rkey][:,self.sim_results[rkey].timepoints[0]]
+            if sim.results[rkey][:].ndim==1:
+                model_output = sim.results[rkey][self.sim_results[rkey].timepoints[0]]
+            else:
+                model_output = sim.results[rkey][:,self.sim_results[rkey].timepoints[0]]
             diffs = self.sim_results[rkey].data.value - model_output
             gofs = hpm.compute_gof(self.sim_results[rkey].data.value, model_output)
             losses = gofs * self.sim_results[rkey].weights
@@ -455,8 +464,8 @@ class Calibration(sc.prettyobj):
             sim_results[rkey] = model_output
 
         extra_sim_results = sc.objdict()
-        if self.extra_sim_results:
-            for rkey in self.extra_sim_results_keys:
+        if self.extra_sim_result_keys:
+            for rkey in self.extra_sim_result_keys:
                 model_output = sim.results[rkey]
                 extra_sim_results[rkey] = model_output
 
@@ -564,6 +573,7 @@ class Calibration(sc.prettyobj):
         # Replace with something else, this is fragile
         self.analyzer_results = []
         self.sim_results = []
+        self.extra_sim_results = []
         if load:
             print('Loading saved results...')
             for trial in study.trials:
@@ -703,9 +713,10 @@ class Calibration(sc.prettyobj):
 
         # Initialize
         fig, axes = pl.subplots(n_rows, n_cols, **fig_args)
-        for ax in axes.flat[n_plots:]:
-            ax.set_visible(False)
-        axes = axes.flatten()
+        if n_plots>1:
+            for ax in axes.flat[n_plots:]:
+                ax.set_visible(False)
+            axes = axes.flatten()
         pl.subplots_adjust(**axis_args)
 
         # Pull out attributes that don't vary by run
@@ -730,7 +741,10 @@ class Calibration(sc.prettyobj):
                 for date in all_dates[rn]:
 
                     # Initialize axis and data storage structures
-                    ax = axes[plot_count]
+                    if n_plots>1:
+                        ax = axes[plot_count]
+                    else:
+                        ax = axes
                     bins = []
                     genotypes = []
                     values = []
@@ -774,35 +788,41 @@ class Calibration(sc.prettyobj):
 
                     # Set title and labels
                     ax.set_xlabel('Age group')
-                    ax.set_title(self.result_args[resname].name+', '+ date.replace('.0', ''))
+                    ax.set_title(f'{self.result_args[resname].name}, {date}')
                     ax.legend()
                     ax.set_xticks(x, age_labels[resname])
                     plot_count += 1
 
             for rn, resname in enumerate(self.sim_results_keys):
-                x = np.arange(len(self.glabels))
-                ax = axes[plot_count]
-                bins = []
-                values = []
+                if n_plots > 1:
+                    ax = axes[plot_count]
+                else:
+                    ax = axes
+                bins = sc.autolist()
+                values = sc.autolist()
                 thisdatadf = self.target_data[rn+sum(dates_per_result)][self.target_data[rn + sum(dates_per_result)].name == resname]
                 ydata = np.array(thisdatadf.value)
+                x = np.arange(len(ydata))
                 ax.scatter(x, ydata, color=pl.cm.Reds(0.95), marker='s', label='Data')
 
-
                 # Construct a dataframe with things in the most logical order for plotting
                 for run_num, run in enumerate(sim_results):
                     bins += x.tolist()
-                    values += list(run[resname])
+                    if sc.isnumber(run[resname]):
+                        values += sc.promotetolist(run[resname])
+                    else:
+                        values += run[resname].tolist()
+
                 # Plot model
                 modeldf = pd.DataFrame({'bins': bins, 'values': values})
                 ax = plot_func(ax=ax, x='bins', y='values', data=modeldf, **extra_args)
 
                 # Set title and labels
                 date = thisdatadf.year[0]
-                ax.set_xlabel('Genotype')
+                # ax.set_xlabel('Genotype')
                 ax.set_title(self.result_args[resname].name + ', ' + str(date))
                 ax.legend()
-                ax.set_xticks(x, self.glabels)
+                # ax.set_xticks(x, self.glabels)
                 plot_count += 1
 
         return hppl.tidy_up(fig, do_save=do_save, fig_path=fig_path, do_show=do_show, args=all_args)
\ No newline at end of file
