@@ -75,7 +75,7 @@ def make_pars(**kwargs):
     pars['n_partner_types'] = 1  # Number of partnership types - reset below
 
     # Basic disease transmission parameters
-    pars['beta']                = 0.21   # Per-act transmission probability; absolute value, calibrated
+    pars['beta']                = 0.2   # Per-act transmission probability; absolute value, calibrated
     pars['transf2m']            = 1.0   # Relative transmissibility of receptive partners in penile-vaginal intercourse; baseline value
     pars['transm2f']            = 3.69  # Relative transmissibility of insertive partners in penile-vaginal intercourse; based on https://doi.org/10.1038/srep10986: "For vaccination types, the risk of male-to-female transmission was higher than that of female-to-male transmission"
     pars['eff_condoms']         = 0.7   # The efficacy of condoms; https://www.nejm.org/doi/10.1056/NEJMoa053284?url_ver=Z39.88-2003&rfr_id=ori:rid:crossref.org&rfr_dat=cr_pub%20%200www.ncbi.nlm.nih.gov
@@ -83,23 +83,27 @@ def make_pars(**kwargs):
     # Parameters for disease progression
     pars['hpv_control_prob']    = 0.0 # Probability that HPV is controlled latently vs. cleared
     pars['hpv_reactivation']    = 0.025 # Placeholder; unused unless hpv_control_prob>0
-    pars['dur_cancer']          = dict(dist='lognormal', par1=12.0, par2=3.0)  # Duration of untreated invasive cerival cancer before death (years)
-    pars['dur_transformed']     = dict(dist='normal_pos', par1=5.77, par2=5)  # Duration of transformed infection prior to onset of invasive cervical cancer (years)
+    pars['dur_cancer']          = dict(dist='lognormal', par1=8.0, par2=3.0)  # Duration of untreated invasive cerival cancer before death (years)
     pars['dur_infection_male']  = dict(dist='lognormal', par1=1, par2=1) # Duration of infection for men
-    pars['clinical_cutoffs']    = dict(precin=0.03, cin1=0.353, cin2=0.676, cin3=0.99) # Parameters used to map disease severity onto cytological grades
-    pars['sev_dist']            = dict(dist='normal_pos', par1=1.0, par2=0.05) # Distribution to draw individual level severity scale factors
+    pars['clinical_cutoffs']    = dict(cin1=0.33, cin2=0.676, cin3=0.8) # Parameters used to map disease severity onto cytological grades
+    pars['sev_dist']            = dict(dist='normal_pos', par1=1.0, par2=0.25) # Distribution to draw individual level severity scale factors
+    pars['age_risk']            = dict(age=30, risk=1)
 
     # Parameters used to calculate immunity
     pars['imm_init']        = dict(dist='beta_mean', par1=0.35, par2=0.025)  # beta distribution for initial level of immunity following infection clearance. Parameters are mean and variance from https://doi.org/10.1093/infdis/jiv753
     pars['imm_decay']       = dict(form=None)  # decay rate, with half life in years
     pars['cell_imm_init']   = dict(dist='beta_mean', par1=0.25, par2=0.025) # beta distribution for level of immunity against persistence/progression of infection following infection clearance and seroconversion
     pars['imm_boost']       = []  # Multiplicative factor applied to a person's immunity levels if they get reinfected. No data on this, assumption.
-    pars['immunity']        = None  # Matrix of immunity and cross-immunity factors, set by init_immunity() in immunity.py
-    pars['cross_imm_med']   = 0.3
-    pars['cross_imm_high']  = 0.5
+    pars['cross_immunity_sus'] = None  # Matrix of susceptibility cross-immunity factors, set by init_immunity() in immunity.py
+    pars['cross_immunity_sev'] = None  # Matrix of severity cross-immunity factors, set by init_immunity() in immunity.py
+    pars['cross_imm_sus_med']   = 0.3
+    pars['cross_imm_sus_high']  = 0.5
+    pars['cross_imm_sev_med']   = 0.5
+    pars['cross_imm_sev_high']  = 0.7
+    pars['own_imm_hr'] = 0.9
 
     # Genotype parameters
-    pars['genotypes']       = [16, 18, 'hrhpv']  # Genotypes to model
+    pars['genotypes']       = [16, 18, 'hi5']  # Genotypes to model
     pars['genotype_pars']   = sc.objdict()  # Can be directly modified by passing in arguments listed in get_genotype_pars
 
     # Events and interventions
@@ -109,12 +113,12 @@ def make_pars(**kwargs):
     pars['stopping_func']   = None # A function to call to stop the sim partway through
 
     # Population distribution of the World Standard Population, used to calculate age-standardised rates (ASR) of incidence
-    pars['age_bins']        = np.array( [  0,   5,  10,  15,  20,  25,  30,  35,  40,  45,  50,  55,  60,  65,  70,  75,    80,    85, 100])
-    pars['standard_pop']    = np.array([pars['age_bins'],
+    pars['age_bin_edges']        = np.array( [  0,   5,  10,  15,  20,  25,  30,  35,  40,  45,  50,  55,  60,  65,  70,  75,    80,    85, 100])
+    pars['standard_pop']    = np.array([pars['age_bin_edges'],
                                         [.12, .10, .09, .09, .08, .08, .06, .06, .06, .06, .05, .04, .04, .03, .02, .01, 0.005, 0.005,   0]])
 
     # The following variables are stored within the pars dict for ease of access, but should not be directly specified.
-    # Rather, they are automaticall constructed during sim initialization.
+    # Rather, they are automatically constructed during sim initialization.
     pars['immunity_map']    = None  # dictionary mapping the index of immune source to the type of immunity (vaccine vs natural)
     pars['imm_kin']         = None  # Constructed during sim initialization using the nab_decay parameters
     pars['genotype_map']    = dict()  # Reverse mapping from number to genotype key
@@ -122,6 +126,7 @@ def make_pars(**kwargs):
     pars['n_imm_sources']   = 1 # The number of immunity sources circulating in the population
     pars['vaccine_pars']    = dict()  # Vaccines that are being used; populated during initialization
     pars['vaccine_map']     = dict()  # Reverse mapping from number to vaccine key
+    pars['cumdysp']         = dict()
 
     # Update with any supplied parameter values and generate things that need to be generated
     pars.update(kwargs)
@@ -153,7 +158,7 @@ def reset_layer_pars(pars, layer_keys=None, force=False):
     layer_defaults['random'] = dict(
         partners    = dict(a=dict(dist='poisson', par1=0.01)), # Everyone in this layer has one partner; this captures *additional* partners. If using a poisson distribution, par1 is roughly equal to the proportion of people with >1 partner
         acts        = dict(a=dict(dist='neg_binomial', par1=100,par2=50)),  # Default number of sexual acts per year for people at sexual peak
-        age_act_pars = dict(a=dict(peak=35, retirement=60, debut_ratio=0.5, retirement_ratio=0.1)), # Parameters describing changes in coital frequency over agent lifespans
+        age_act_pars = dict(a=dict(peak=35, retirement=100, debut_ratio=0.5, retirement_ratio=0.1)), # Parameters describing changes in coital frequency over agent lifespans
         layer_probs = dict(a=1.0),  # Default proportion of the population in each layer
         dur_pship   = dict(a=dict(dist='normal_pos', par1=5,par2=3)),    # Default duration of partnerships
         condoms     = dict(a=0.25),  # Default proportion of acts in which condoms are used
@@ -168,10 +173,10 @@ def reset_layer_pars(pars, layer_keys=None, force=False):
         acts         = dict(m=dict(dist='neg_binomial', par1=80, par2=40), # Default number of acts per year for people at sexual peak
                             c=dict(dist='neg_binomial', par1=10, par2=5), # Default number of acts per year for people at sexual peak
                             o=dict(dist='neg_binomial', par1=1,  par2=.01)),  # Default number of acts per year for people at sexual peak
-        age_act_pars = dict(m=dict(peak=30, retirement=60, debut_ratio=0.5, retirement_ratio=0.1), # Parameters describing changes in coital frequency over agent lifespans
-                            c=dict(peak=25, retirement=60, debut_ratio=0.5, retirement_ratio=0.1),
-                            o=dict(peak=25, retirement=50, debut_ratio=0.5, retirement_ratio=0.1)),
-        dur_pship   = dict(m=dict(dist='normal_pos', par1=20, par2=3),
+        age_act_pars = dict(m=dict(peak=30, retirement=100, debut_ratio=0.5, retirement_ratio=0.1), # Parameters describing changes in coital frequency over agent lifespans
+                            c=dict(peak=25, retirement=100, debut_ratio=0.5, retirement_ratio=0.1),
+                            o=dict(peak=25, retirement=100, debut_ratio=0.5, retirement_ratio=0.1)),
+        dur_pship   = dict(m=dict(dist='normal_pos', par1=12, par2=3),
                            c=dict(dist='normal_pos', par1=1, par2=1),
                            o=dict(dist='normal_pos', par1=0.1, par2=0.05)),
         condoms     = dict(m=0.01, c=0.2, o=0.1),  # Default proportion of acts in which condoms are used
@@ -248,9 +253,12 @@ def get_genotype_choices():
     '''
     # List of choices available
     choices = {
-        'hpv16':  ['hpv16', '16'],
-        'hpv18': ['hpv18', '18'],
-        'hrhpv': ['hrhpv', 'ohrhpv', 'hr', 'ohr'],
+        'hpv16':    ['hpv16', '16'],
+        'hpv18':    ['hpv18', '18'],
+        'hi5':      ['hi5hpv', 'hi5hpv', 'cross-protective'],
+        'ohr':      ['ohrhpv', 'non-cross-protective'],
+        'hr':       ['allhr', 'allhrhpv', 'hrhpv', 'oncogenic', 'hr10', 'hi10'],
+        'lo':       ['lohpv'],
     }
     mapping = {name:key for key,synonyms in choices.items() for name in synonyms} # Flip from key:value to value:key
     return choices, mapping
@@ -284,22 +292,6 @@ def get_vaccine_choices():
     return choices, mapping
 
 
-
-def get_treatment_choices():
-    '''
-    Define valid pre-defined treatment names
-    '''
-    # List of choices currently available: new ones can be added to the list along with their aliases
-    choices = {
-        'default': ['default', None],
-        'ablative':  ['ablative', 'thermal_ablation', 'TA'],
-        'excisional': ['excisional', 'leep'],
-        'radiation': ['radiation']
-    }
-    mapping = {name:key for key,synonyms in choices.items() for name in synonyms} # Flip from key:value to value:key
-    return choices, mapping
-
-
 def _get_from_pars(pars, default=False, key=None, defaultkey='default'):
     ''' Helper function to get the right output from genotype functions '''
 
@@ -328,30 +320,68 @@ def get_genotype_pars(default=False, genotype=None):
     pars = sc.objdict()
 
     pars.hpv16 = sc.objdict()
-    pars.hpv16.dur_episomal     = dict(dist='lognormal', par1=4.5, par2=9) # Duration of episomal infection prior to cancer
-    pars.hpv16.sev_fn           = dict(form='logf3', k=0.3, x_infl=13, s=1) # Function mapping duration of infection to severity
+    pars.hpv16.dur_precin       = dict(dist='normal_pos', par1=0.5, par2=0.25)  # Duration of infection prior to precancer
+    pars.hpv16.dur_episomal     = dict(dist='lognormal', par1=2, par2=5) # Duration of episomal infection prior to cancer
+    pars.hpv16.sev_fn           = dict(form='logf2', k=0.175, x_infl=0, ttc=30) # Function mapping duration of infection to severity
     pars.hpv16.rel_beta         = 1.0  # Baseline relative transmissibility, other genotypes are relative to this
-    pars.hpv16.transform_prob   = 0.00025 # Annual rate of transformed cell invading
+    pars.hpv16.transform_prob   = 1.3e-9 # Annual rate of transformed cell invading
+    pars.hpv16.sev_integral     = 'analytic' # Type of integral used for translating severity to transformation probability. Accepts numeric, analytic, or None
     pars.hpv16.sero_prob        = 0.75 # https://www.sciencedirect.com/science/article/pii/S2666679022000027#fig1
 
     pars.hpv18 = sc.objdict()
-    pars.hpv18.dur_episomal     = dict(dist='lognormal', par1=3.5, par2=9) # Duration of infection prior to cancer
-    pars.hpv18.sev_fn           = dict(form='logf3', k=0.238, x_infl=14, s=1) # Function mapping duration of infection to severity
+    pars.hpv18.dur_precin       = dict(dist='normal_pos', par1=0.5, par2=0.25)  # Duration of infection prior to precancer
+    pars.hpv18.dur_episomal     = dict(dist='lognormal', par1=2, par2=5) # Duration of infection prior to cancer
+    pars.hpv18.sev_fn           = dict(form='logf2', k=0.15, x_infl=0, ttc=30) # Function mapping duration of infection to severity
     pars.hpv18.rel_beta         = 0.75  # Relative transmissibility, current estimate from Harvard model calibration of m2f tx
-    pars.hpv18.transform_prob   = 0.00015 # Annual rate of transformed cell invading
+    pars.hpv18.transform_prob   = 1.0e-9 # Annual rate of transformed cell invading
+    pars.hpv18.sev_integral     = 'analytic' # Type of integral used for translating severity to transformation probability. Accepts numeric, analytic, or None
     pars.hpv18.sero_prob        = 0.56 # https://www.sciencedirect.com/science/article/pii/S2666679022000027#fig1
 
-    pars.hrhpv = sc.objdict()
-    pars.hrhpv.dur_episomal     = dict(dist='lognormal', par1=5, par2=10) # Duration of infection prior to cancer
-    pars.hrhpv.sev_fn           = dict(form='logf3', k=0.35, x_infl=15, s=1) # Function mapping duration of infection to severity
-    pars.hrhpv.rel_beta         = 0.9 # placeholder
-    pars.hrhpv.transform_prob   = 0.00015
-    pars.hrhpv.sero_prob        = 0.60 # placeholder
+    # High-risk oncogenic types included in 9valent vaccine: 31, 33, 45, 52, 58
+    pars.hi5 = sc.objdict()
+    pars.hi5.dur_precin         = dict(dist='normal_pos', par1=0.5, par2=0.25)  # Duration of infection prior to precancer
+    pars.hi5.dur_episomal       = dict(dist='lognormal', par1=2, par2=4) # Duration of infection prior to cancer
+    pars.hi5.sev_fn             = dict(form='logf2', k=0.125, x_infl=0, ttc=30) # Function mapping duration of infection to severity
+    pars.hi5.rel_beta           = 0.9 # placeholder
+    pars.hi5.transform_prob     = 3e-10 # Annual rate of transformed cell invading
+    pars.hi5.sev_integral       = 'analytic' # Type of integral used for translating severity to transformation probability. Accepts numeric, analytic, or None
+    pars.hi5.sero_prob          = 0.60 # placeholder
+
+    # Other high-risk: oncogenic but not covered in 9valent vaccine: 35, 39, 51, 56, 59
+    pars.ohr = sc.objdict()
+    pars.ohr.dur_precin         = dict(dist='normal_pos', par1=0.5, par2=0.25)  # Duration of infection prior to precancer
+    pars.ohr.dur_episomal       = dict(dist='lognormal', par1=2, par2=6) # Duration of infection prior to cancer
+    pars.ohr.sev_fn             = dict(form='logf2', k=0.125, x_infl=0, ttc=30) # Function mapping duration of infection to severity
+    pars.ohr.rel_beta           = 0.9 # placeholder
+    pars.ohr.transform_prob     = 3e-10 # Annual rate of transformed cell invading
+    pars.ohr.sev_integral       = 'analytic' # Type of integral used for translating severity to transformation probability. Accepts numeric, analytic, or None
+    pars.ohr.sero_prob          = 0.60 # placeholder
+
+    # All other high-risk types: 31, 33, 35, 39, 45, 51, 52, 56, 58, 59
+    # Warning: this should not be used in conjuction with hi5 or ohr
+    pars.hr = sc.objdict()
+    pars.hr.dur_precin       = dict(dist='normal_pos', par1=0.5, par2=0.25)  # Duration of infection prior to precancer
+    pars.hr.dur_episomal     = dict(dist='lognormal', par1=2, par2=4) # Duration of infection prior to cancer
+    pars.hr.sev_fn           = dict(form='logf2', k=0.125, x_infl=0, ttc=30) # Function mapping duration of infection to severity
+    pars.hr.rel_beta         = 0.9 # placeholder
+    pars.hr.transform_prob   = 3e-10 # Annual rate of transformed cell invading
+    pars.hr.sev_integral     = 'analytic' # Type of integral used for translating severity to transformation probability. Accepts numeric, analytic, or None
+    pars.hr.sero_prob        = 0.60 # placeholder
+
+    # Low-risk
+    pars.lr = sc.objdict()
+    pars.lr.dur_precin          = dict(dist='normal_pos', par1=0.5, par2=0.25)  # Duration of infection prior to precancer
+    pars.lr.dur_episomal        = dict(dist='lognormal', par1=2, par2=4) # Duration of infection prior to cancer
+    pars.lr.sev_fn              = dict(form='logf2', k=0.0, x_infl=0, ttc=30) # Function mapping duration of infection to severity
+    pars.lr.rel_beta            = 0.9 # placeholder
+    pars.lr.transform_prob      = 0 # Annual rate of transformed cell invading
+    pars.lr.sev_integral        = 'analytic' # Type of integral used for translating severity to transformation probability. Accepts numeric, analytic, or None
+    pars.lr.sero_prob           = 0.60 # placeholder
 
     return _get_from_pars(pars, default, key=genotype, defaultkey='hpv16')
 
 
-def get_cross_immunity(cross_imm_med=None, cross_imm_high=None, default=False, genotype=None):
+def get_cross_immunity(cross_imm_med=None, cross_imm_high=None, own_imm_hr=None, default=False, genotype=None):
     '''
     Get the cross immunity between each genotype in a sim
     '''
@@ -360,19 +390,46 @@ def get_cross_immunity(cross_imm_med=None, cross_imm_high=None, default=False, g
         hpv16 = dict(
             hpv16=1.0, # Default for own-immunity
             hpv18=cross_imm_high,
-            hrhpv=cross_imm_med,
+            hi5=cross_imm_med,
+            ohr=cross_imm_med,
+            hr=cross_imm_med,
+            lr=cross_imm_med,
         ),
 
         hpv18 = dict(
             hpv16=cross_imm_high,
             hpv18=1.0,  # Default for own-immunity
-            hrhpv=cross_imm_med,
+            hi5=cross_imm_med,
+            ohr=cross_imm_med,
+            hr=cross_imm_med,
+            lr=cross_imm_med,
         ),
 
-        hrhpv=dict(
+        hi5=dict(
             hpv16=cross_imm_med,
             hpv18=cross_imm_med,
-            hrhpv=cross_imm_med,
+            hi5=own_imm_hr,
+            ohr=cross_imm_med,
+            hr=cross_imm_med,
+            lr=cross_imm_med,
+        ),
+
+        ohr=dict(
+            hpv16=cross_imm_med,
+            hpv18=cross_imm_med,
+            hi5=cross_imm_med,
+            ohr=own_imm_hr,
+            hr=cross_imm_med,
+            lr=cross_imm_med,
+        ),
+
+        lr=dict(
+            hpv16=cross_imm_med,
+            hpv18=cross_imm_med,
+            hi5=cross_imm_med,
+            ohr=cross_imm_med,
+            hr=cross_imm_med,
+            lr=own_imm_hr,
         ),
 
     )
@@ -642,3 +699,38 @@ def compute_inv_severity(sev_vals, rel_sev=None, pars=None):
         output = output / rel_sev
 
     return output
+
+
+def compute_severity_integral(t, rel_sev=None, pars=None):
+    '''
+    Process functional form and parameters into values:
+    '''
+
+    pars = sc.dcp(pars)
+    form = pars.pop('form')
+    choices = [
+        'logf2',
+        'logf3 with s=1',
+    ]
+
+    # Scale t
+    if rel_sev is not None:
+        t = rel_sev * t
+
+    # Process inputs
+    if form is None or form == 'logf2':
+        output = hpu.intlogf2(t, **pars)
+
+    elif form=='logf3':
+        s = pars.pop('s')
+        if s==1:
+            output = hpu.intlogf2(t, **pars)
+        else:
+            errormsg = f'Analytic integral for logf3 only implemented for s=1. Select integral=numeric.'
+
+    else:
+        errormsg = f'Analytic integral for the selected functional form "{form}" is not implemented; choices are: {sc.strjoin(choices)}, or select integral=numeric.'
+        raise NotImplementedError(errormsg)
+
+    return output
+
